###
###
### Masterarbeit
###
### Titel der Arbeit: Distribution modeling of paleofauna in the Western Mediterranean between
### the Henrich events H5 and H4
###
### Michael Holthausen, 24.09. 2018
### mholthausen@icloud.com
###
### Dieses Script wurde für die Datenverarbeitung zur Masterarbeit verwendet.
### Für Spezies ohne Sampling des Bias und SSB
###

#############################################################################
# Libraries                                                                 #
#############################################################################
library(sp)
library(raster)
library(dismo)
library(usdm)
library(jsonlite)
library(maptools)
library(rgdal)
library(rJava)
library(XML)
library(rgeos)

source('SDM_Utils.R')

#############################################################################
# Set the current working directory,                                        #
# file paths and ext-Variable                                               #
#############################################################################
# Current Working Directory
cwd <- getwd()

# Set current Data Directory 
cdd <- "/Users/micha/Documents/Masterarbeit"          #macOS

# Set path to Climate Data Files
cdf <- paste(cdd,"/Climate_Data", sep="")

# Set path to Species Data
sd <- paste(cdd,"/species", sep="")

# Suffix for extended model extent
extended <- "_ext"

# Suffix for MaxEnt-Folder
maxent_suffix <- "_maxent"

#############################################################################
# Environment                                                               #
#############################################################################
# Folder names of time slices
time_slices <- c('H4_ext','GIS11_ext','H5_ext','recent_ext')

# select the index of time_slices to be modeled
ts <- 3

# Set the extent
ext <- extent(-12,20,28,60)

# research area extent
ra_ext <- extent(-12,4,28,44)

#############################################################################
# Transformation of the climate data                                        #
# !Only activate if bio not calculated already!                             #
#############################################################################
# for (i in seq(1, length(time_slices), 1)) {
# biovarsClimateData(time_slices[i])
# }

#############################################################################
# Patch recent bioclim                                                     #
# !Only execute if there is no recent bioclim!                             #
#############################################################################
# r <- raster()
# r[] <- 1:ncell(r)
# ra <- aggregate(r, 10)
# r1 <- crop(r, extent(-12,20, 28, 60))
# projection(r1) <- CRS("+proj=longlat +datum=WGS84 +no_defs +ellps=WGS84 +towgs84=0,0,0")
# plot(r1)
# r1
# res(r1) <- 0.05
# r1[] <- 1:ncell(r1)
# r1
# 
# for (i in 1:19) {
#  file <- raster(paste("/Users/micha/Documents/Masterarbeit/01_Worldclim_Bioclim/Present/bio/bio_",i,"/w001001.adf",sep = ''))
#  file2_crop <- crop(file, extent(r1))
#  projection(file2_crop) <- CRS("+proj=longlat +datum=WGS84 +no_defs +ellps=WGS84 +towgs84=0,0,0")
#  #file2_res <- resample(file, raster(paste(cdf,'/H4/bio/bio1.tif', sep='')), resample='bilinear')
#  file2_res <- resample(file2_crop, r1, resample='bilinear')
#  #file2_crop <- crop(file2_res, extent(raster(paste(cdf,'/H4/bio/bio1.tif', sep=''))))
#  #writeRaster(file2_crop, paste('/Users/micha/Documents/Masterarbeit/Climate_Data/recent/bio/bio',i,'.tif', sep=''), format="GTiff", overwrite=TRUE)
#  writeRaster(file2_res, paste('/Users/micha/Documents/Masterarbeit/Climate_Data/recent_ext/bio/bio',i,'.tif', sep=''), format="GTiff", overwrite=TRUE)
# }
#

#############################################################################
# Data preperation                                                         #
# List of species and selection of the species to be calculated             #
#############################################################################
species.list <- read.csv2(file='Species_List_Modeling.csv', header = TRUE, sep =';', stringsAsFactors=FALSE)
species.list

# Select the index of species to be modeled
species.sel <- 7
species.name <- strsplit(species.list[species.sel,1], " ")
species.fullname <- species.list$Species[species.sel]
species.name.sub <- gsub(" ", "_", species.fullname, fixed=TRUE)

# Check for MaxEnt-Folder, if not existent, create it
if (file.exists(paste(cdd,"/Results/maps/models/",species.name.sub,maxent_suffix,sep=""))){
  print("OK")
} else {
  dir.create(file.path(paste(cdd,"/Results/maps/models/",species.name.sub,maxent_suffix,sep="")))
}

# Start the Logfile
log_con <- file(paste(cdd,"/Results/maps/models/", "LOGFILE_", species.name.sub,".txt",sep=""), open="w")
cat("Start: ", file = log_con, sep="\n")
cat("Species: " , species.fullname, file = log_con, sep="")
cat("", file = log_con, sep="\n")
cat("Time: " , time_slices[ts], file = log_con, sep="")
cat("", file = log_con, sep="\n")
sys_start <- Sys.time()
cat(format(Sys.time(), "%a %b %d %X %Y"), file = log_con, sep="")

#############################################################################
# Occurrence data: Read from prepared csv                                   #
# id = gbif database id, x = longitude, y = latitude                        #
#############################################################################
file <- paste(sd, "/", species.name.sub, extended, ".csv", sep="")
species.data <- read.table(file, header = TRUE, sep = ";")

# Set Column names
colnames(species.data)[2] <- "lon"
colnames(species.data)[3] <- "lat"

head(species.data)

#############################################################################
# Create a simple outline map from GADM-Dataset                             #
#############################################################################
study_area.map <- readOGR(paste(cdd, "/GADM/merged", extended, ".shp", sep=""), layer = paste("merged", extended, sep=""))

#############################################################################
# Clean up the data: remove duplicates                                      #
#############################################################################
dups2 <- duplicated(species.data[, c( 'lon' ,  'lat' )])

# number of duplicates
sum(dups2)
species.clean <- species.data[!dups2, ]

#############################################################################
# Cross-checking of coordinates                                             #
#############################################################################
coordinates(species.clean) <- ~lon + lat
crs(species.clean) <- crs("+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs +towgs84=0,0,0")
class(species.clean)
species.clean

#############################################################################
# Spatial query of the coordinates                                          #
#############################################################################
ovr <- over(species.clean, study_area.map, fn = NULL)
head(ovr)
ovr

# Remove finding spots which are not on land
cntr <- ovr$NAME_ISO
i <- which(is.na(cntr))
i

if(length(i) > 0) {
  dim(species.clean)
  species.clean <- species.clean[-i, ]
  dim(species.clean)
}

attributes(cntr)

png(paste(cdd, "/Results/maps/presence_data/", species.name.sub, extended, "_presence", ".png", sep=""), width = 1280, height = 1280, units = "px", pointsize = 18, bg = "white",  res = NA)
plot(study_area.map, main=paste("GBIF Presence data:", species.fullname, sep=" "), xlim=c(-12,20), ylim=c(28,60), axes=TRUE)
box()
points(species.data$lon, species.data$lat, col='orange', pch=20, cex=0.75)
points(species.data$lon, species.data$lat, col='red', cex=0.75)
dev.off()

cat("", file = log_con, sep="\n")
cat("Cleaned data: " , dim(species.clean))

#############################################################################
# Sampling bias                                                             #
#############################################################################

cat("", file = log_con, sep="\n")
cat("Sampling Grid Resolution: " , "NO sampling, as occurrence data is NO bias", file = log_con, sep="")
species.sampl <- species.clean

file <- paste(cdd, "/Results/maps/data/", species.name.sub, extended, "_sampledData", ".csv", sep="")
write.csv2(species.sampl, file = file)
species.sampl <- read.csv2(file)

species.sampl

cat("", file = log_con, sep="\n")
cat("Input sampled data: " , nrow(species.sampl), file = log_con, sep="")

coordinates(species.sampl) <- ~lon+lat
projection(species.sampl) <- CRS('+proj=longlat +datum=WGS84')
class(species.sampl)

png(paste(cdd, "/Results/maps/presence_data/", species.name.sub, extended, "_sampling", "_",time_slices[ts], ".png", sep=""), width = 1280, height = 1280, units = "px", pointsize = 18, bg = "white",  res = NA)
plot(study_area.map, main=paste("GBIF Presence data (sampled):", species.fullname, sep=" "), xlim=c(-12,20), ylim=c(28,60), axes=TRUE)
box()
points(species.sampl$lon, species.sampl$lat, col='orange', pch=20, cex=0.75)
points(species.sampl$lon, species.sampl$lat, col='red', cex=0.75)
dev.off()

#############################################################################
# Environment data                                                          #
#############################################################################

#############################################################################
# Create recent data stack                                                  #
#############################################################################
files_r <- list.files(path=paste(cdf, "/recent",extended,"/bio",sep=""), pattern='.tif', full.names=TRUE)
paste(cdf, "/recent",extended,"/bio",sep="")
files_r <- append(files_r, paste(cdd, '/GEBCO/recent',extended,'/GebCo_TB_ext.tif', sep=''))
files_r <- append(files_r, paste(cdd, '/GEBCO/recent',extended,'/GebCo_TB_ext_slope.tif', sep=''))
files_r <- append(files_r, paste(cdd, '/GEBCO/recent',extended,'/GebCo_TB_ext_aspect.tif', sep=''))
files_r <- append(files_r, paste(cdd, '/GEBCO/recent',extended,'/iwahashi.tif', sep=''))
files_r <- append(files_r, paste(cdf, '/recent',extended,'/kg/kg.tif', sep=''))

files_r
predictors_recent <- stack(files_r)

#############################################################################
# Add LandClassification, GebCo and bio for past predictors                 #
#############################################################################

files_p <- list.files(path=paste(cdf,'/',time_slices[ts],'/bio', sep=''), pattern='.tif', full.names=TRUE)
files_p <- append(files_p, paste(cdd, '/GEBCO/',time_slices[ts],'/GebCo_TB_ext.tif', sep=''))
files_p <- append(files_p, paste(cdd, '/GEBCO/',time_slices[ts],'/GebCo_TB_ext_slope.tif', sep=''))
files_p <- append(files_p, paste(cdd, '/GEBCO/',time_slices[ts],'/GebCo_TB_ext_aspect.tif', sep=''))
files_p <- append(files_p, paste(cdd, '/GEBCO/',time_slices[ts],'/iwahashi.tif', sep=''))
files_p <- append(files_p, paste(cdf, '/',time_slices[ts],'/kg/KG.tif', sep=''))

files_p
predictors <- stack(files_p)


#############################################################################
#visually investigate colinearity in the environmental data
#use only ecological meaningful variables
#############################################################################
 png(paste(cdd, "/Results/maps/models/", species.name.sub, "_pairs_plot_sdmdata", "_", time_slices[ts], ".png", sep=""), width = 1280, height = 1280, units = "px", pointsize = 18, bg = "white",  res = NA)
 pairs(sdmdata[,2:5], cex=0.1, fig=TRUE)
 dev.off()

vif(predictors_recent)
names(predictors_recent)
v1 <- vifstep(predictors_recent, th=10)
v1

# reduce the stack to ecological significant biovars (see Literature: Beaumont et al. (2016))

predictors_recent <- dropLayer(predictors_recent, "bio2")
predictors_recent <- dropLayer(predictors_recent, "bio3")
predictors_recent <- dropLayer(predictors_recent, "bio7")
predictors_recent <- dropLayer(predictors_recent, "bio8")
predictors_recent <- dropLayer(predictors_recent, "bio9")
predictors_recent <- dropLayer(predictors_recent, "bio10")
predictors_recent <- dropLayer(predictors_recent, "bio11")
predictors_recent <- dropLayer(predictors_recent, "bio16")
predictors_recent <- dropLayer(predictors_recent, "bio17")
predictors_recent <- dropLayer(predictors_recent, "bio18")
predictors_recent <- dropLayer(predictors_recent, "bio19")

predictors <- dropLayer(predictors, "bio2")
predictors <- dropLayer(predictors, "bio3")
predictors <- dropLayer(predictors, "bio7")
predictors <- dropLayer(predictors, "bio8")
predictors <- dropLayer(predictors, "bio9")
predictors <- dropLayer(predictors, "bio10")
predictors <- dropLayer(predictors, "bio11")
predictors <- dropLayer(predictors, "bio16")
predictors <- dropLayer(predictors, "bio17")
predictors <- dropLayer(predictors, "bio18")
predictors <- dropLayer(predictors, "bio19")
v1 <- vif(predictors_recent)

cat("\n",  file = log_con)
cat("VIF predictors_recent: " , "",  file = log_con, sep="\n")
cat( v1$Variables,  file = log_con, sep=" ")
cat( v1$VIF,  file = log_con, sep=" ")
cat("\n",  file = log_con)

names(predictors)
names(predictors_recent)
#############################################################################
# Presence and backgrounddata                                               #
# k-fold data partioning                                                    #
#############################################################################

#############################################################################
# Remove spatial sorting bias, to compare AUC                               #
# Background data for training and a testing set                            #
#############################################################################

file <- paste(cdd, "/Results/maps/data/", species.name.sub, extended, "_sampledData", ".csv", sep="")
species.sampl <- read.csv2(file)[,3:4] #[, 2:3]
presvals <- extract(predictors_recent, species.sampl)
set.seed(1963)
backg <- randomPoints(predictors_recent, n=1000, ext=ext)
colnames(backg) = c( 'lon' ,  'lat' )
absvals <- extract(predictors_recent, backg)
pb <- c(rep(1, nrow(presvals)), rep(0, nrow(absvals)))
sdmdata <- data.frame(cbind(pb, rbind(presvals, absvals)))
sdmdata[, 'iwahashi' ] = as.factor(sdmdata[, 'iwahashi' ])
sdmdata[, 'KG' ] = as.factor(sdmdata[, 'KG' ])

pred_nf <- dropLayer(predictors_recent,  'iwahashi' )
pred_nf <- dropLayer(pred_nf,  'KG' )
pred_nf_past <- dropLayer(predictors,  'iwahashi' )
pred_nf_past <- dropLayer(pred_nf_past,  'KG' )

k <- 5
group <- kfold(species.sampl, 5)
pres_train <- species.sampl[group != 1, ]
pres_test <- species.sampl[group == 1, ]

group <- kfold(backg, 5)
backg_train <- backg[group != 1, ]
backg_test <- backg[group == 1, ]

r = raster(pred_nf, 1)
png(paste(cdd, "/Results/maps/presence_data/", species.name.sub, "_backgpres_train_test", "_", time_slices[ts], ".png", sep=""), width = 1280, height = 1280, units = "px", pointsize = 18, bg = "white",  res = NA)
plot(!is.na(r), col=c('white', 'light grey'), legend=FALSE)
plot(ext, add=TRUE, col='red', lwd=2)
points(backg_train, pch='-', cex=0.5, col='red')
points(backg_test, pch='-', cex=0.5, col='black')
points(pres_train, pch='+', col='green')
points(pres_test, pch='+', col='blue')
legend("topleft", legend=c("- Background (training)", "- Background (test)", "+ Presence (trainig)", "+ Presence (test)"), text.col=c("red", "black", "green", "blue"), cex=0.8, bg="grey")
dev.off()


## BIOCLIM

bc <- bioclim(pred_nf, pres_train)
e <- evaluate(pres_test, backg_test, bc, pred_nf)
e
tr <- threshold(e, 'spec_sens')
tr

pb <- predict(pred_nf_past, bc, ext=ra_ext, progress='')

png(paste(cdd, "/Results/maps/models/", species.name.sub, "_bioclim_response", "_", time_slices[ts], ".png", sep=""), width = 1280, height = 1280, units = "px", pointsize = 18, bg = "white",  res = NA)
response(bc)
dev.off()

png(paste(cdd, "/Results/maps/models/", species.name.sub, "_bioclim_pairs", "_", time_slices[ts], ".png", sep=""), width = 1280, height = 1280, units = "px", pointsize = 18, bg = "white",  res = NA)
pairs(bc)
dev.off()

png(paste(cdd, "/Results/maps/models/", species.name.sub, "_bioclim_eval", "_", time_slices[ts], ".png", sep=""), width = 1280, height = 1280, units = "px", pointsize = 18, bg = "white",  res = NA)
par(mfrow=c(1, 2))
density(e)
boxplot(e, col=c( 'blue' ,  'red' ), notch =FALSE)
dev.off()

png(paste(cdd, "/Results/maps/models/", species.name.sub, "_bioclim_auc", "_", time_slices[ts], ".png", sep=""), width = 1280, height = 1280, units = "px", pointsize = 18, bg = "white",  res = NA)
plot(e,  'ROC' )
dev.off()

cat("", file = log_con, sep="\n")
cat("### BIOCLIM ###: ", file = log_con, sep="\n")
cat(paste("n presence training (pres_train)", "\t", " : ", nrow(backg_test), sep=""), file = log_con, sep="\n")
cat(paste("n presences (pres_test)", "\t", " : ", e@np, sep=""), file = log_con, sep="\n")
cat(paste("n absences (back_test)", "\t", " : ", e@na, sep=""), file = log_con, sep="\n")
cat(paste("AUC", "\t\t\t", " : ", e@auc, sep=""), file = log_con, sep="\n")
cat(paste("cor", "\t", " : ", e@cor, sep=""), file = log_con, sep="\n")
cat(paste("Threshold", "\t", " : ", tr, sep=""), file = log_con, sep="\n")


png(paste(cdd, "/Results/maps/models/", species.name.sub, "_bioclim", "_", time_slices[ts], ".png", sep=""), width = 1280, height = 1280, units = "px", pointsize = 18, bg = "white",  res = NA)
par(mfrow=c(1,2))
plot(pb, main='Bioclim, raw values')
plot(study_area.map, add=TRUE)
plot(pb > tr, main='presence/absence')
plot(study_area.map, add=TRUE)
points(pres_train, pch='+')
points(backg_train, pch= "-" , cex=0.25)
dev.off()

writeRaster(pb, paste(cdd,"/Results/maps/models/tif/",species.name.sub,"_bioclim", "_", time_slices[ts], ".tif", sep=""),  format="GTiff", overwrite=TRUE)
writeRaster(pb>tr, paste(cdd,"/Results/maps/models/tif/",species.name.sub,"_bioclim", "_", time_slices[ts], "_th.tif", sep=""),  format="GTiff", overwrite=TRUE)

##regression preparation
train <- rbind(pres_train, backg_train)
pb_train <- c(rep(1, nrow(pres_train)), rep(0, nrow(backg_train)))
envtrain <- extract(predictors_recent, train)
envtrain <- data.frame( cbind(pa=pb_train, envtrain))

envtrain[, 'iwahashi' ] = factor(envtrain[, 'iwahashi' ], levels=c(3,19,35,51,65,66,67,81,82,83,97,98,99,113,114,115,255))
envtrain[, 'KG' ] = factor(envtrain[, 'KG' ], levels=1:31)
head(envtrain)
testpres <- data.frame( extract(predictors_recent, pres_test) )
testbackg <- data.frame( extract(predictors_recent, backg_test) )
testbackg[ , 'iwahashi' ] = factor(testbackg[ , 'iwahashi' ], levels=c(3,19,35,51,65,66,67,81,82,83,97,98,99,113,114,115,255))
testbackg[ , 'KG' ] = factor(testbackg[ , 'KG' ], levels=1:31)
envtrain

## GLM
gm1 <- glm(pa ~ bio1 + bio4 + bio5 + bio6 + bio12 + bio15 + bio13 + bio14 + GebCo_TB_ext + GebCo_TB_ext_slope + GebCo_TB_ext_aspect, family = binomial(link = "logit"), data=envtrain)
summary(gm1)

coef(gm1)
ge1 <- evaluate(testpres, testbackg, gm1)
ge1
tr <- threshold(e, 'spec_sens')
tr
pg <- predict(predictors, gm1, ext=ra_ext)

png(paste(cdd, "/Results/maps/models/", species.name.sub, "_glm_eval", "_", time_slices[ts], ".png", sep=""), width = 1280, height = 1280, units = "px", pointsize = 18, bg = "white",  res = NA)
par(mfrow=c(1, 2))
density(ge1)
boxplot(ge1, col=c( 'blue' ,  'red' ), notch = FALSE)
dev.off()

png(paste(cdd, "/Results/maps/models/", species.name.sub, "_glm_auc", "_", time_slices[ts], ".png", sep=""), width = 1280, height = 1280, units = "px", pointsize = 18, bg = "white",  res = NA)
plot(ge1,  'ROC' )
dev.off()

png(paste(cdd, "/Results/maps/models/", species.name.sub, "_glm_response", "_", time_slices[ts], ".png", sep=""), width = 1280, height = 1280, units = "px", pointsize = 18, bg = "white",  res = NA)
response(gm1)
dev.off()

cat("", file = log_con, sep="\n")
cat("### GLM ###: ", file = log_con, sep="\n")
cat(paste("n presences", "\t", " : ", ge1@np, sep=""), file = log_con, sep="\n")
cat(paste("n absences", "\t", " : ", ge1@na, sep=""), file = log_con, sep="\n")
cat(paste("AUC", "\t\t\t", " : ", ge1@auc, sep=""), file = log_con, sep="\n")
cat(paste("cor", "\t", " : ", ge1@cor, sep=""), file = log_con, sep="\n")
cat(paste("Threshold", "\t", " : ", tr, sep=""), file = log_con, sep="\n")

cat("", file = log_con, sep="\n")
cat(paste("Continous predictors:", "Intercept; bio1 + bio4 + bio5 + bio6 + bio12 + bio15 + bio13 + bio14 + GebCo_TB_ext + GebCo_TB_ext_slope + GebCo_TB_ext_aspect", sep=""), file = log_con, sep=" ")

cat("", file = log_con, sep="\n")
cat(paste("Coefs:", coef(gm1), sep=""), file = log_con, sep="\n")

png(paste(cdd, "/Results/maps/models/", species.name.sub, "_glm", "_", time_slices[ts], ".png", sep=""), width = 960, height = 960, units = "px", pointsize = 18, bg = "white",  res = NA)
par(mfrow=c(1,2))
plot(pg, main="GLM/binomial, raw values")
plot(study_area.map, add=TRUE)
plot(pg > tr, main="presence/absence")
plot(study_area.map, add=TRUE)
points(pres_train, pch= "+" )
points(backg_train, pch= "-" , cex=0.25)
dev.off()

writeRaster(pg, paste(cdd,"/Results/maps/models/tif/",species.name.sub,"_glm", "_", time_slices[ts],".tif", sep=""),  format="GTiff", overwrite=TRUE)
writeRaster(pg>tr, paste(cdd,"/Results/maps/models/tif/",species.name.sub,"_glm", "_", time_slices[ts],"_th.tif", sep=""),  format="GTiff", overwrite=TRUE)


# MAXENT
jar <- paste(system.file(package="dismo"), "/java/maxent.jar", sep=''  )
xm <- maxent(predictors_recent, pres_train, factors=c("iwahashi", "KG"), path=paste(cdd, "/Results/maps/models/",species.name.sub, maxent_suffix, sep=""))
e <- evaluate(pres_test, backg_test, xm, predictors_recent)
e
tr <- threshold(e, 'spec_sens')
tr
px <- predict(predictors, xm, ext=ra_ext, progress='')

png(paste(cdd, "/Results/maps/models/", species.name.sub, "_maxent_eval", "_", time_slices[ts], ".png", sep=""), width = 1280, height = 1280, units = "px", pointsize = 18, bg = "white",  res = NA)
par(mfrow=c(1, 2))
density(e)
boxplot(e, col=c( 'blue' ,  'red' ), notch=FALSE)
dev.off()

png(paste(cdd, "/Results/maps/models/", species.name.sub, "_maxent_auc", "_", time_slices[ts], ".png", sep=""), width = 1280, height = 1280, units = "px", pointsize = 18, bg = "white",  res = NA)
plot(e,  'ROC' )
dev.off()

png(paste(cdd, "/Results/maps/models/", species.name.sub, "_maxent_response", "_", time_slices[ts], ".png", sep=""), width = 1280, height = 1280, units = "px", pointsize = 18, bg = "white",  res = NA)
response(xm)
dev.off()

png(paste(cdd, "/Results/maps/models/", species.name.sub, "_maxent_pairs", "_", time_slices[ts], ".png", sep=""), width = 1280, height = 1280, units = "px", pointsize = 18, bg = "white",  res = NA)
pairs(xm)
dev.off()

cat("", file = log_con, sep="\n")
cat("### MAXENT ###: ", file = log_con, sep="\n")
cat(paste("n presences", "\t", " : ", e@np, sep=""), file = log_con, sep="\n")
cat(paste("n absences", "\t", " : ", e@na, sep=""), file = log_con, sep="\n")
cat(paste("AUC", "\t\t\t", " : ", e@auc, sep=""), file = log_con, sep="\n")
cat(paste("cor", "\t", " : ", e@cor, sep=""), file = log_con, sep="\n")
cat(paste("Threshold", "\t", " : ", tr, sep=""), file = log_con, sep="\n")

png(paste(cdd, "/Results/maps/models/", species.name.sub, "_maxent", "_", time_slices[ts], ".png", sep=""), width = 960, height = 960, units = "px", pointsize = 18, bg = "white",  res = NA)
par(mfrow=c(1,2))
plot(px, main='Maxent, raw values')
plot(study_area.map, add=TRUE, border='dark grey')
plot(px > tr, main='presence/absence')
plot(study_area.map, add=TRUE, border='dark grey')
points(pres_train, pch='+')
points(backg_train, pch= "-" , cex=0.25)
dev.off()

writeRaster(px, paste(cdd,"/Results/maps/models/tif/",species.name.sub,"_maxent", "_", time_slices[ts],".tif", sep=""),  format="GTiff", overwrite=TRUE)
writeRaster(px>tr, paste(cdd,"/Results/maps/models/tif/",species.name.sub,"_maxent", "_", time_slices[ts],"_th.tif", sep=""),  format="GTiff", overwrite=TRUE)

# End the Logfile
cat("", file = log_con, sep="\n")
cat("Stop: ", file = log_con, sep="")
cat(format(Sys.time(), "%a %b %d %X %Y"), file = log_con, sep="\n")
sys_end <- Sys.time()

duration <- sys_end - sys_start
cat("", file = log_con, sep="\n")
cat("Duration: ", duration, " minutes", file = log_con, sep="")
close(log_con)
